local AceOO = AceLibrary("AceOO-2.0");
if (not Yata.Class) then Yata.Class = {} end;
Yata.Class.Button = AceOO.Class("AceEvent-2.0");
Yata.Class.Button.virtual = true;
Yata.Class.IButton = AceOO.Interface{Assign = "function", Show = "function", Hide = "function"};
Yata.Class.ControlButton = AceOO.Class(Yata.Class.Button, Yata.Class.IButton);
Yata.Class.StandardButton = AceOO.Class(Yata.Class.Button, Yata.Class.IButton);
Yata.Class.Header = AceOO.Class();

---------------------------
-- Button Template stuff --
---------------------------

-- Show stuff
function Yata.Class.Button.prototype:Show()
	if(self.Frame) then self.Frame:Show(); end
end
-- Hide stuff
function Yata.Class.Button.prototype:Hide()
	if(self.Frame) then self.Frame:Hide(); end
end
function Yata.Class.Button.prototype:Assign(totem)
	if(self.Frame) then
		local btexture = getglobal(self.Frame:GetName().."Icon");
		btexture:SetTexture(totem.Texture);
		self.Totem = totem;
		self.Frame:SetAttribute("*type*", "spell");
		self.Frame:SetAttribute("spell1", self.Totem.Name);
		self.Frame:SetAttribute("spell2", Yata.L["Totemic Call"]);
		if(#Yata.TotemData[totem.Name] > 1) then
			local spellName, spellRank = GetSpellName((Yata.TotemData[totem.Name])[1].SpellID, BOOKTYPE_SPELL);
			self.Frame:SetAttribute("shift-spell1",spellName.."("..spellRank..")");
		end
	end
end

function Yata.Class.Button.prototype:SetTooltip()
	local frame = self.Frame;
	if ( GetCVar("UberTooltips") == "1" ) then
		GameTooltip_SetDefaultAnchor(GameTooltip, frame);
	else
		GameTooltip:SetOwner(frame, "ANCHOR_RIGHT");
	end
	GameTooltip:SetSpell( self.Totem.SpellID, BOOKTYPE_SPELL );
end

--------------------
-- Control Button --
--------------------
function Yata.Class.ControlButton.prototype:init(ele, ID, header)
	Yata.Class.ControlButton.super.prototype.init(self);
	self.Frame = getglobal("YATABUTTON"..string.upper(ele)..ID);
	if(not self.Frame) then
		self.Frame = CreateFrame("Button", "YATABUTTON"..string.upper(ele)..ID, header.Frame, "ActionButtonTemplate,SecureActionButtonTemplate,SecureAnchorEnterTemplate");
	end
	self.ButtonID = ID;
	self.IsShown = true;
	self.Frame:RegisterForClicks("LeftButtonUp","RightButtonUp");
	self.Frame:RegisterForDrag( "LeftButton" );
	header.Frame:SetAttribute("AddChild",self.Frame);
	self.Frame:SetAttribute("showstates", "*");
	self.Frame:SetAttribute("delaystate", "0");
	self.Frame:SetAttribute("delaytime", "0.5");
	self.Frame:SetAttribute("delayhover", "any");
	self.Frame:SetAttribute("newstate", tostring(ID));
	self.Frame:SetAttribute("*childraise-OnEnter", true);
	self.Frame:SetAttribute("*childstate-OnEnter", "enter"..ID);
	self.Frame:SetAttribute("*childstate-OnLeave", "leave"..ID);
	self.Frame:SetAttribute("anchorchild", header.Frame);
	self.Frame:SetAttribute("childstate", "^" .. tostring(ID));
	self.Frame:SetScript("PostClick",function() self:TriggerEvent("Yata_ButtonClick",ID); end);
	self.Frame:SetScript("OnDragStart", function() Yata.TotemBar:ButtonDrag(true, ID); end);
	self.Frame:SetScript("OnReceiveDrag", function() Yata.TotemBar:ButtonDrag(nil, ID); end);
	-- Necessary tricks for tooltips
	self.Frame.ParentButton = self;
end

-----------
-- Hooks --
-----------

function Yata.Class.ControlButton.prototype:ControlOnEnter(type)
	if(not Yata:CheckShowToolTip() or not Yata.IsInitialized or not self.ParentButton) then return end;
	if(type == "OnEnter") then
		self.ParentButton:SetTooltip();
	elseif(type == "OnLeave") then
		GameTooltip:Hide();
	end
end

-------------------
-- Normal Button --
-------------------


function Yata.Class.StandardButton.prototype:init(ele,ID,header)
	Yata.Class.StandardButton.super.prototype.init(self);
	self.Frame = getglobal("YATABUTTON"..string.upper(ele)..ID);
	if(not self.Frame) then
		self.Frame = CreateFrame("Button", "YATABUTTON"..string.upper(ele)..ID, self.Header, "SecureActionButtonTemplate,ActionButtonTemplate");
	end
	self.ButtonID = ID;
	self.IsShown = true;
	self.Frame:RegisterForDrag( "LeftButton" );
	self.Frame:RegisterForClicks("LeftButtonUp","RightButtonUp");
	self.Frame:SetAttribute("showstates", math.floor(ID/10));
	self.Frame:SetAttribute("newstate", "0");
	self.Frame:SetScript("PostClick",function() self:TriggerEvent("Yata_ButtonClick",ID); end);
	self.Frame:SetScript("OnEnter", function() if(Yata:CheckShowToolTip()) then self:SetTooltip(ID) end end);
	self.Frame:SetScript("OnLeave", function() if(Yata:CheckShowToolTip()) then GameTooltip:Hide(); end end);
	self.Frame:SetScript("OnDragStart", function() Yata.TotemBar:ButtonDrag(true, ID); end);
	self.Frame:SetScript("OnReceiveDrag", function() Yata.TotemBar:ButtonDrag(nil, ID); end);
	local border = getglobal(self.Frame:GetName().."Border");
	border:SetVertexColor(0, 1.0, 0, 0.5);
	header.Frame:SetAttribute("AddChild",self.Frame);
	-- Necessary tricks for tooltips
	self.Frame.ParentButton = self;
end


------------
-- Header --
------------

function Yata.Class.Header.prototype:init()
	Yata.Class.Header.super.prototype.init(self);
	self.Frame = CreateFrame("Frame", "YATAHEADERFRAME",  UIParent, "SecureStateDriverTemplate");
	self.Frame:SetAttribute("statemap-Totem", "$input");
	self.Frame:SetWidth(20);
	self.Frame:SetHeight(38);
	self.Frame.Texture = self.Frame:CreateTexture();
	self.Frame.Texture:SetTexture(0, 0, 0.5, 1);
	self.Frame.Texture:SetAllPoints(self.Frame);
	self.Frame.Texture:Hide();
	for i = 1, 4 do
		self.Frame:SetAttribute("statemap-anchor-enter"..i, "*:"..i);
		self.Frame:SetAttribute("statemap-anchor-leave"..i,      ";");
		self.Frame:SetAttribute("delaystatemap-anchor-leave"..i, i..":0");
		self.Frame:SetAttribute("delaytimemap-anchor-leave"..i,  i..":0.5");
		self.Frame:SetAttribute("delayhovermap-anchor-leave"..i, i..":true");
	end
	self.Frame:Show();
	self.Frame:RegisterForDrag( "LeftButton" );
end
